// DayOfWeek.java
//
// ICS 22 / CSE 22 Fall 2012
// Code Example
//
// An enum provides a way of creating a new type of object which has three
// interesting properties:
//
// (1) There is a small, finite number of objects of this type that will ever
//     need to be created.
// (2) We know already, before the program is written, which objects will be
//     needed.
// (3) The set of objects that will be needed is not expected to change.
//
// When we have this set of circumstances, an enum provides a short, simple
// way to get what we're looking for.
//
// The days of the week provide a useful example.  In a program that needs to
// store and/or manipulate day-of-week values, there will only ever need to
// be seven possible days of the week: Sunday, Monday, Tuesday, Wednesday,
// Thursday, Friday, and Saturday.  We know all of these days already, and
// we know they won't change over time.  So an enum is appropriate here.
//
// It's important to realize that an enum is, in a lot of ways, like a class.
// Enums, like classes, define a new kind of object.  Enums can have methods,
// fields, and constructors.  The only thing they can't have are *public*
// constructors, meaning that it is not possible to create new objects of
// this type, beyond the small, constant set that we define as part of the
// enum.  So, for days of the week, we'll set up the seven objects ahead of
// time, and will never permit new days of the week to be created.

public enum DayOfWeek
{
	// In an enum, we first list the names of the objects that make up the
	// enum.  By convention, these objects are named like other constants
	// in Java, by using all uppercase, with words separated by underscores.
	//
	// If all we want is the set of objects, with no methods, fields, or
	// constructors, we only need to list the names, separated by commas,
	// with no semicolon at the end.  A semicolon is used to separate the
	// list of names from methods, fields, and constructors.
	//
	// In this case, I'm also calling a constructor (written below) on each
	// of the DayOfWeek objects, passing a String parameter to it.  My goal
	// is for each DayOfWeek object to know its own shorthand abbreviation,
	// so I can ask it "What's your abbreviation?" and get back an answer.
	
	SUNDAY ("Sun"),
	MONDAY ("Mon"),
	TUESDAY ("Tue"),
	WEDNESDAY ("Wed"),
	THURSDAY ("Thu"),
	FRIDAY ("Fri"),
	SATURDAY ("Sat");


	// It may seem strange to have a private constructor -- which is a
	// constructor that can only be called within the class (or, in this
	// case, enum) that it's declared.  But this is actually the perfect
	// tool here: provide the ability for the DayOfWeek objects being
	// created within the enum to be initialized, while not allowing
	// code anywhere else in the program to create new DayOfWeek objects.
	private DayOfWeek(String abbreviation)
	{
		this.abbreviation = abbreviation;
	}


	// This field stores the abbreviation associated with a DayOfWeek object.
	private String abbreviation;


	// getAbbreviation() returns the abbreviation associated with this
	// DayOfWeek object.
	public String getAbbreviation()
	{
		return abbreviation;
	}
	
	
	// The next() method returns the day of the week that follows "this" one
	// (i.e., the one that you call the next() method on).  So if you call
	// the next() method on the DayOfWeek.TUESDAY object, you'll get back
	// DayOfWeek.WEDNESDAY.
	//
	// Note that we're using the names here without qualification (i.e., we're
	// saying "SUNDAY" instead of "DayOfWeek.SUNDAY").  That's because the code
	// is being written in the DayOfWeek enum, so SUNDAY has meaning in this
	// scope.  Everywhere else, we'll have to say DayOfWeek.SUNDAY.  (The only
	// exception to this rule is in the cases of a switch statement, which
	// curiously require you not to qualify the names, no matter where the
	// switch statement is written.)
	public DayOfWeek next()
	{
		switch (this)
		{
		case SUNDAY:
			return MONDAY;
			
		case MONDAY:
			return TUESDAY;
			
		case TUESDAY:
			return WEDNESDAY;
			
		case WEDNESDAY:
			return THURSDAY;
			
		case THURSDAY:
			return FRIDAY;
			
		case FRIDAY:
			return SATURDAY;
			
		case SATURDAY:
			return SUNDAY;

		// We can reasonably expect this never to happen, but still have to
		// write it, because the compiler will otherwise complain that it's
		// possible that no value will be returned from the method.			
		default:
			throw new RuntimeException("How has this happened?");
		}
	}
}
